package de.intevation.flys.exports.minfo;

import org.apache.log4j.Logger;
import org.jfree.data.xy.XYSeries;
import org.w3c.dom.Document;

import de.intevation.artifactdatabase.state.ArtifactAndFacet;
import de.intevation.artifactdatabase.state.Facet;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.minfo.BedDiffEpochResult;
import de.intevation.flys.exports.StyledSeriesBuilder;
import de.intevation.flys.exports.XYChartGenerator;
import de.intevation.flys.jfree.FLYSAnnotation;
import de.intevation.flys.jfree.StyledXYSeries;


public class BedDifferenceEpochGenerator
extends XYChartGenerator
implements FacetTypes
{
    public enum YAXIS {
        D(0), H(1);

        protected int idx;

        private YAXIS(int c) {
            idx = c;
        }
    }

    /** The logger that is used in this generator. */
    private static Logger logger = Logger.getLogger(BedQualityGenerator.class);

    public static final String I18N_CHART_TITLE = "chart.beddifference.epoch.title";
    public static final String I18N_XAXIS_LABEL = "chart.beddifference.xaxis.label";
    public static final String I18N_YAXIS_LABEL = "chart.beddifference.yaxis.label.diff";
    public static final String I18N_SECOND_YAXIS_LABEL = "chart.beddifference.yaxis.label.height";

    public static final String I18N_CHART_TITLE_DEFAULT = "Sohlenhöhen Differenz";
    public static final String I18N_XAXIS_LABEL_DEFAULT = "Fluss-Km";
    public static final String I18N_YAXIS_LABEL_DEFAULT = "delta S [m]";
    public static final String I18N_SECOND_YAXIS_LABEL_DEFAULT = "Höhe [m]";

    @Override
    protected YAxisWalker getYAxisWalker() {
        return new YAxisWalker() {

            @Override
            public int length() {
                return YAXIS.values().length;
            }

            @Override
            public String getId(int idx) {
                YAXIS[] yaxes = YAXIS.values();
                return yaxes[idx].toString();
            }
        };
    }

    @Override
    public void doOut(ArtifactAndFacet bundle, Document attr, boolean visible) {
        String name = bundle.getFacetName();

        logger.debug("doOut: " + name);

        if (name == null) {
            logger.error("No facet name for doOut(). No output generated!");
            return;
        }

        Facet facet = bundle.getFacet();

        if (facet == null) {
            return;
        }

        if (name.equals(BED_DIFFERENCE_EPOCH)) {
            doBedDifferenceEpochOut(
                (BedDiffEpochResult) bundle.getData(context),
                bundle, attr, visible);
        }
        else if (name.equals(BED_DIFFERENCE_EPOCH_HEIGHT1)) {
            doBedDifferenceHeightsOut((BedDiffEpochResult)bundle.getData(context),
                bundle, attr, visible, 0);
        }
        else if (name.equals(BED_DIFFERENCE_EPOCH_HEIGHT2)) {
            doBedDifferenceHeightsOut((BedDiffEpochResult)bundle.getData(context),
                bundle, attr, visible, 1);
        }
        else if (name.equals(LONGITUDINAL_ANNOTATION)) {
            doAnnotations(
                (FLYSAnnotation) bundle.getData(context),
                 bundle,
                 attr,
                 visible);
        }
    }

    @Override
    protected String getDefaultChartTitle() {
        return msg(I18N_CHART_TITLE, I18N_CHART_TITLE_DEFAULT);
    }

    @Override
    protected String getDefaultXAxisLabel() {
        return msg(I18N_XAXIS_LABEL, I18N_XAXIS_LABEL_DEFAULT);
    }

    @Override
    protected String getDefaultYAxisLabel(int pos) {
        String label = "default";
        if (pos == YAXIS.D.idx) {
            label = msg(I18N_YAXIS_LABEL, I18N_YAXIS_LABEL_DEFAULT);
        }
        else if (pos == YAXIS.H.idx) {
            label = msg(I18N_SECOND_YAXIS_LABEL, I18N_SECOND_YAXIS_LABEL_DEFAULT);
        }
 
        return label;
    }

    protected void doBedDifferenceEpochOut(BedDiffEpochResult data,
        ArtifactAndFacet aandf, Document theme, boolean visible) {

        XYSeries series = new StyledXYSeries(aandf.getFacetDescription(), theme);
        StyledSeriesBuilder.addPoints(series, data.getDifferencesData(), true);

        addAxisSeries(series, YAXIS.D.idx, visible);
    }

    private void doBedDifferenceHeightsOut(
        BedDiffEpochResult data,
        ArtifactAndFacet bundle,
        Document attr,
        boolean visible,
        int idx) {
         logger.debug("doBedDifferenceHeightOut()");

        XYSeries series = new StyledXYSeries(bundle.getFacetDescription(), attr);
        if (idx == 0) {
            StyledSeriesBuilder.addPoints(series, data.getHeights1Data(), true);
        }
        else {
            StyledSeriesBuilder.addPoints(series, data.getHeights2Data(), true);
        }

        addAxisSeries(series, YAXIS.H.idx, visible);
    }

}
